/* eslint-disable @typescript-eslint/no-unused-vars */
import React from 'react';
import { Button, Layout, Menu, List, Dropdown, Badge, Avatar, Upload, message, Input, Drawer, Modal } from 'antd'
import _get from 'lodash/get'
import { ImgMsg } from './components/MsgCard/ImgMsg';
import { TextMsg } from './components/MsgCard/TextMsg';
import { GroupSetting } from './components/GroupSetting';
import './Chat.css';
declare const AIM: any;
const {
    IMSDK, // 核心 SDK
    DataSDK, // 数据 SDK
    constants, // 常量定义
    convFeature, // 会话相关辅助函数
    msgFeature, // 消息相关辅助函数
    initStore, // 数据 SDK，初始化 Store
    getFileUrl,  // 获取文件访问地址
    mediaId, // 资源 ID 工具,
    ConnectionStatus,
    EventNameEnum,
    IMsgStatusType,
    IRecallOperatorType,
    TypingType,
} = AIM

const MSG_PAGE_SIZE = 20;

function Chat() {
    const [store, setStore] = React.useState(initStore);
    const { convIdList, convMap, msgIdList, msgMap, typingStatusMap, appCid } = store
    const [inputText, setInputText] = React.useState<string>('');
    const mgsListRef = React.useRef<HTMLDivElement>(null);
    const isTyping: boolean = !!appCid && typingStatusMap[appCid] !== undefined;
    const conversationType = convMap[appCid] ? convFeature.getConversationType(convMap[appCid]) : 1;
    const [groupSettingVisible, setGroupSettingVisible] = React.useState('');
    React.useEffect(() => {
        const storeUnsubscribe = DataSDK.subscribe((newStore: any) => {
            console.log('store 更新', newStore)
            setStore(newStore);
        });
        DataSDK.listConversations(0, 1000); // 拉取首屏，最多1000， 最近会话超出1000将不显示，不支持分页
        // IMSDK.getAuthService().setDomain('apitest')
        // DataSDK.activeConv('$3$55556666:850000000')
        // @
        return () => {
            storeUnsubscribe();
        };
    }, []);
    const scrollToBottom = React.useCallback(() => {
        setTimeout(() => {
            const $msgList = mgsListRef.current;
            if ($msgList && $msgList.firstElementChild) {
                $msgList.firstElementChild.scrollTop = $msgList.firstElementChild.scrollHeight;
            }
        }, 0);
    }, []);

    const scrollToMessage = React.useCallback((mid, offset = 0) => {
        const $msgList = mgsListRef.current;
        const element: HTMLLIElement | null = document.querySelector(`#message-${mid.replace('.', '_')}`);
        if ($msgList && $msgList.firstElementChild && element) {
            $msgList.firstElementChild.scrollTop = element.offsetTop + offset;
        }
    }, []);

    const sendMsg = React.useCallback(() => {
        if (appCid) {
            DataSDK.sendTextMessage(appCid, inputText);
            setInputText('');
        }
    }, [appCid, inputText]);

    React.useEffect(() => {
        IMSDK.getEventService().register((eventName: string) => {
            if (eventName === EventNameEnum.PaaSMessageAdd) {
                scrollToBottom();
            }
        });
    }, []);
    // 全部禁言
    const isSilenceAll = convMap[appCid]?.groupChatUserConversation?.groupChatConversation?.silenceAll === 1
    // 被单独禁言
    const isSilence = convMap[appCid]?.groupChatUserConversation?.silencedStatus === 1 && convMap[appCid]?.groupChatUserConversation?.silencedEndTime > 0
    console.log('store', store)
    return (
        <Layout className="layout">
            <Layout.Sider theme="light" width={360}>
                <List
                    size="default"
                    style={{ height: '100%', overflow: 'auto' }}
                    bordered
                    dataSource={convIdList}
                    renderItem={(cid: string) => {
                        const conv = convMap[cid];
                        const isGroupConv = conv.type === 2;
                        const isTop = (conv.singleChatUserConversation?.topRank || 0) > 0 ||
                            (conv.groupChatUserConversation?.topRank || 0) > 0;
                        const isMute = conv.singleChatUserConversation?.muteNotification === 1 ||
                            conv.groupChatUserConversation?.muteNotification === 1;
                        const isVisable = conv.singleChatUserConversation?.visible === 1 ||
                            conv.groupChatUserConversation?.visible === 1;
                        const iconMid = conv?.groupChatUserConversation?.groupChatConversation?.icon
                        let icon = 'https://zos.alipayobjects.com/rmsportal/ODTLcjxAfvqbxHnVXCYX.png'

                        if (iconMid && mediaId.isMediaId(iconMid)) {
                            try {
                                icon = mediaId.mid2Url(iconMid, {
                                    imageSize: 'origin',
                                    queryParams: {
                                        domain: IMSDK.getAuthService().getDomain() || '',
                                    },
                                })
                            } catch (ex) {
                                console.log('xxx', ex)
                            }
                        }

                        const menu = (
                            <Menu>
                                <Menu.Item key="1" onClick={() => {
                                    IMSDK.getConvService().setTop(cid, !isTop, convFeature.getConversationType(conv));
                                }}>{isTop ? '取消置顶' : '置顶'}</Menu.Item>
                                <Menu.Item key="2" onClick={() => {
                                    IMSDK.getConvService().mute(cid, !isMute, convFeature.getConversationType(conv));
                                }}>{isMute ? '取消免打扰' : '免打扰'}</Menu.Item>
                                <Menu.Item key="3" onClick={() => {
                                    if (isVisable) {
                                        IMSDK.getConvService().hide(cid, convFeature.getConversationType(conv));
                                    }
                                }}>{isVisable ? '移除' : '已移除，不可见'}</Menu.Item>
                                <Menu.Item key="4" onClick={() => {
                                    IMSDK.getConvService().clear(cid, convFeature.getConversationType(conv));
                                }}>清空会话</Menu.Item>
                                {isGroupConv && (
                                    <Menu.Item key="5" onClick={() => {
                                        if (conv.groupChatUserConversation?.groupChatConversation?.silenceAll === 1) {

                                            IMSDK.getConvService().groupConversationCancelSilenceAll(cid).then(() => {
                                                message.info('已解除禁言')
                                            }).catch((ex: any) => {
                                                if (ex?.body.reason) {
                                                    message.error(ex?.body.reason);
                                                }
                                            })
                                        } else {
                                            IMSDK.getConvService().groupConversationSilenceAll(cid).then(() => {
                                                message.info('已开始禁言')
                                            }).catch((ex: any) => {
                                                if (ex?.body.reason) {
                                                    message.error(ex?.body.reason);
                                                }
                                            })
                                        }
                                    }}>{conv.groupChatUserConversation?.groupChatConversation?.silenceAll === 1 ? '解除禁言' : '开启禁言'}</Menu.Item>
                                )}
                            </Menu>
                        );
                        let description = '';
                        const lastMsg = conv.singleChatUserConversation?.lastMessage || conv.groupChatUserConversation?.lastMessage;
                        if (lastMsg) {
                            description = lastMsg?.message?.content?.text?.content || '';
                            if (lastMsg?.message?.content?.contentType === 2) {
                                description = '[图片]'
                            }
                            if (lastMsg.msgStatus === IMsgStatusType.recall && lastMsg.recallFeature?.showRecallStatusSetting) {
                                description = getRecallText(lastMsg, msgFeature.isMyMsg(lastMsg));
                            }
                        }
                        const redPoint = convFeature.isSingleConv(conv) ? conv.singleChatUserConversation?.redPoint : conv.groupChatUserConversation?.redPoint;
                        return (

                            <List.Item className={`conv-item ${cid === appCid ? 'active' : ''}`}
                                style={{ opacity: isVisable ? 1 : 0.2 }}>
                                <div
                                    style={{ flexGrow: 1 }}
                                    onClick={() => {
                                        console.log(1111, cid);
                                        DataSDK.activeConv(cid).then(scrollToBottom);
                                    }}>
                                    <List.Item.Meta
                                        avatar={
                                            <Avatar src={icon} />
                                        }
                                        title={<Badge count={redPoint || 0}><span style={{
                                            textOverflow: 'ellipsis',
                                            maxWidth: 160,
                                            overflow: 'hidden',
                                            display: 'inline-block',
                                            whiteSpace: 'nowrap',
                                        }}>{conv.type === 2 ? `[群聊]${conv.groupChatUserConversation.groupChatConversation.title}` : `[单聊]${cid}`}</span></Badge>}
                                        description={description}
                                    />

                                </div>
                                <div>
                                    <Dropdown
                                        overlay={menu}
                                        trigger={['hover']}
                                    >
                                        <small className="conv-title-btn">More</small>
                                    </Dropdown>
                                    &nbsp;
                                    <small className="conv-title-btn" onClick={() => {
                                        setGroupSettingVisible(cid)
                                    }}>Setting</small>
                                </div>
                            </List.Item>
                        );
                    }}
                />
            </Layout.Sider>

            <Layout.Content className="chat-main">
                <div className="msg-list" ref={mgsListRef}>
                    <List
                        id="msg-list-container"
                        size="default"
                        style={{ overflow: 'auto', height: '100%' }}
                        bordered
                        locale={{ emptyText: !!appCid && msgIdList.length === 0 ? '暂无消息' : '欢迎使用' }}
                        dataSource={msgIdList}
                        header={(msgIdList.length > 0 && store.msgPrevHasMore === true) ? <div onClick={() => {
                            const mid = msgIdList[0];
                            const cursor = store.msgPrevCursor;
                            if (cursor) {
                                DataSDK.listPrevMsgs(`${cursor}`, MSG_PAGE_SIZE).then(() => {
                                    scrollToMessage(mid, -20);
                                });
                            }
                        }} style={{ textAlign: 'center' }}><Button type="default" size="small">加载上一页</Button></div> :
                            !!appCid && <div style={{ textAlign: 'center' }}><Button type="default" size="small" disabled>消息已全部加载</Button>
                            </div>}
                        renderItem={(mid: string) => {
                            return (
                                <MsgContent msg={msgMap[mid]} conversationType={conversationType} />
                            );
                        }}
                    />
                </div>
                <div className="sendbox">
                    <div className="topBar">
                        <Upload
                            name="file"
                            showUploadList={false}
                            disabled={!appCid || isSilenceAll || isSilence}
                            multiple={true}
                            accept=".jpg, .jpeg, .png, .gif"
                            beforeUpload={(file) => {
                                DataSDK.sendPhotoMessage(appCid, file).catch((ex: any) => {
                                    console.log('上传出错了', ex)
                                })
                                return false;
                            }}>
                            <Button>+ 图片</Button>
                        </Upload>
                        {isTyping && <text style={{ fontSize: 12, marginLeft: '1em', display: 'inline-block' }}>对方正在输入...</text>}
                    </div>
                    <Input.TextArea
                        className="sendbox-text"
                        value={inputText}
                        disabled={!appCid || isSilenceAll || isSilence}
                        placeholder={isSilenceAll ? '群主已开启禁言' :
                            `${isSilence ? `禁言中，结束时间 ${convMap[appCid]?.groupChatUserConversation?.silencedEndTime}` : ''}`
                        }
                        onChange={(e) => {
                            setInputText(e.target.value || '');
                        }}
                        onFocus={() => {
                            DataSDK.updateTypingStatus(true, TypingType.text);
                        }}
                        onBlur={() => {
                            DataSDK.updateTypingStatus(false);
                        }} />
                    <Button onClick={sendMsg} className="sendbtn" type="primary" disabled={!appCid || !inputText.trim() || isSilenceAll || isSilence}>发送</Button>
                </div>
            </Layout.Content>
            <GroupSetting close={() => {
                setGroupSettingVisible('')
            }} appCid={groupSettingVisible} conv={convMap[groupSettingVisible]} />
        </Layout>
    );
}

export default Chat;

function getRecallText(msg: any, isMyMsg: boolean) {
    // 撤回类型: 发送者撤回：0 ，群主撤回：1，系统撤回：2，安全撤回：3
    const recallType = msg.recallFeature?.operatorType;
    const recalltext: string = ({
        [`${IRecallOperatorType.sender}`]: `${isMyMsg ? '你' : `${msg.message?.sender?.uid || _get(msg, 'message.sender.uid')}`}撤回了消息`,
        [`${IRecallOperatorType.groupOwner}`]: '群主撤回了消息',
    })[`${recallType}`];
    return recalltext || '消息已撤回';
};




export const MsgContent: React.FunctionComponent<{ msg: any; conversationType: 1 | 2 }> = React.memo(({ msg, conversationType }) => {
    const cid = _get(msg, 'message.appCid')
    const messageId = msg.message?.messageId;
    const isMyMsg = msgFeature.isMyMsg(msg);
    const isTextMsg = msg.message?.content?.contentType === 1;
    const isImgMsg = msg.message?.content?.contentType === 2;
    const isCustomMsg = msg.message?.content?.contentType === 101;
    // msgStatus 消息状态。0-删除,1-正常, 2-撤回，3-指定接收者删除, 4-消息降级(拉首屏专用)
    // 目前被删除的消息 不会给客户端。 服务端 msgStatus 有点问题
    if (msg.msgStatus === IMsgStatusType.deleted) {
        return (
            <List.Item className={'msg-content'} style={{ alignItems: 'center' }}>
                消息已删除
            </List.Item>
        );
    } else if (msg.msgStatus === IMsgStatusType.recall && msg.recallFeature?.showRecallStatusSetting) {
        return (
            <List.Item className={'msg-content'} style={{ alignItems: 'center' }}>
                {getRecallText(msg, isMyMsg)}
            </List.Item>
        );
    }
    const isSingle = conversationType === 1;
    const unreadCount = msg.message?.msgReadStatusSetting === 1 && msg.message?.unreadCount || 0;
    let myMsgReadStatusText = `${unreadCount === 0 ? '已读' : '未读'}`;
    if (!isSingle) {
        // 群里
        myMsgReadStatusText = `${unreadCount === 0 ? '全部' : `${unreadCount}人`}${myMsgReadStatusText}`;
    }

    const menu = (
        <Menu>
            {isMyMsg && <Menu.Item key="recall" onClick={() => {
                IMSDK.getMsgService().recallMessage(cid || '', messageId || '', conversationType).catch((ex: any) => {
                    message.error(ex?.body?.reason || '撤回失败');
                });
            }}>撤回</Menu.Item>}
            {!isMyMsg && (
                msg.readStatus === 2 ?
                    <Menu.Item key="readed">已经读了</Menu.Item> :
                    (
                        <Menu.Item key="read" onClick={() => {
                            IMSDK.getMsgService().updateMessageToRead(cid || '', [messageId || ''], conversationType);
                        }}>读消息</Menu.Item>
                    )
            )}
            <Menu.Item key="delete" onClick={() => {
                IMSDK.getMsgService().deleteMessage(cid || '', messageId || '', conversationType);
            }}>删除</Menu.Item>
        </Menu>
    );
    let msgContent: JSX.Element | null = null;
    switch (msg.message?.content?.contentType || 0) {
        case 1:
            msgContent = <TextMsg msg={msg} />;
            break;
        case 2:
            msgContent = <ImgMsg msg={msg} />;
            break;
    }
    return (
        <List.Item id={`message-${messageId?.replace('.', '_') || ''}`} className={`msg-content ${isMyMsg ? 'my-msg' : ''}`}>
            <div className="msg-content" style={{ whiteSpace: 'pre' }}>
                {msgContent}
            </div>
            {isImgMsg && msg.userExtension?.uploadProgress && msg.userExtension?.uploadProgress !== '100' && <span>上传进度：{msg.userExtension?.uploadProgress}%</span>}
            {isMyMsg && !!cid && !!messageId && (
                <small style={{ cursor: 'pointer' }} onClick={() => {
                    IMSDK.getMsgService().ListMessageReadStatus(cid, messageId).then((res: any[]) => {
                        message.info(
                            <div style={{ textAlign: 'left' }}>
                                {res.map((item) => <div key={item.uid}>{`${item.uid}: ${item.status === 2 ? '已读' : '未读'}`}</div>)}
                            </div>,
                        );
                    });
                }}>{myMsgReadStatusText}</small>
            )}

            {isCustomMsg && <span>图片： {msg.message?.content?.custom?.title} 进度：{msg.message?.content?.custom?.data}</span>}
            {!isMyMsg && <span>读过消息：{msg.readStatus === 2 ? 'true' : 'false'}</span>}
            <small>Sender：{msg.message?.sender}</small>
            {!!msg.message?.createAt && <small>createAt：{new Date(msg.message.createAt).toLocaleString()}</small>}
            <small>MsgId：{msg.message?.messageId || msg.message?.uuid}</small>
            <Dropdown
                overlay={menu}
                trigger={['click']}
            >
                <small>设置</small>
            </Dropdown>
            <br />
        </List.Item>
    );
});
