import React, { useState, useCallback, FunctionComponent, useEffect } from 'react';
import { Button, Dropdown, Input, Menu, Modal, Form, Drawer, message, Tag, Upload, Divider, Select } from 'antd';

import './Topbar.less';


declare const AIM: any;
const FormItem = Form.Item;
const {
    IMSDK, // 核心 SDK
    DataSDK, // 数据 SDK
    constants, // 常量定义
    convFeature, // 会话相关辅助函数
    msgFeature, // 消息相关辅助函数
    initStore, // 数据 SDK，初始化 Store
    getFileUrl,  // 获取文件访问地址
    mediaId, // 资源 ID 工具,
    ConnectionStatus,
    EventNameEnum,
    FileUpload,
} = AIM

export interface IProps {
    conv: any;
    appCid: string;
    close: () => void;
}

enum ModalType {
    singleConv,
    groupConv,
}

// 成员角色。取值：
// 1：群主
// 2：管理员
// 3：普通
// 100~127：自定义
const GroupRoleMap: { [key: number]: string } = {
    1: '群主',
    2: '管理员',
    3: '成员'
}

interface IconUploadData {
    progress?: number;
    base64?: string;
    fileName?: string;
}

export const GroupSetting: FunctionComponent<IProps> = React.memo((props: IProps) => {
    const { conv, appCid, close } = props;
    const [groupMembers, setGroupMembers] = React.useState([])
    const [iconUpload, setIconUpload] = React.useState<IconUploadData>({})
    const isGroupConv = conv?.type === 2
    React.useEffect(() => {
        if (isGroupConv) {
            IMSDK.getConvService().groupConversationListAllMembers(appCid).then((res: any) => {
                if (res && res.length > 0) {
                    setGroupMembers(res)
                }
            }).catch((ex: any) => {
                if (ex?.body.reason) {
                    message.error(ex?.body.reason);
                }
            })
        } else {
            setGroupMembers([])
        }
    }, [conv, appCid])

    const updateTitle = useCallback((e: any) => {
        e.preventDefault();
        const newTitle = prompt('输入群名称', conv.groupChatUserConversation.groupChatConversation.title)
        if (newTitle) {
            IMSDK.getConvService().groupConversationUpdateTitle(appCid, newTitle).then(() => {
                message.info('群名称修改成功')
            }).catch((ex: any) => {
                if (ex?.body.reason) {
                    message.error(ex?.body.reason);
                }
            })
        }
    }, [appCid])

    const updateIcon = useCallback((e: any) => {
        e.preventDefault();
        const newIcon = prompt('输入群图标', conv.groupChatUserConversation.groupChatConversation.icon)
        if (newIcon) {
            IMSDK.getConvService().groupConversationUpdateIcon(appCid, newIcon).then(() => {
                message.info('群图标修改成功')
            }).catch((ex: any) => {
                if (ex?.body.reason) {
                    message.error(ex?.body.reason);
                }
            })
        }
    }, [appCid])
    let icon = ''
    const iconMid = conv?.groupChatUserConversation?.groupChatConversation?.icon
    if (iconMid && mediaId.isMediaId(iconMid)) {
        try {
            icon = mediaId.mid2Url(iconMid, {
                imageSize: 'origin',
                queryParams: {
                    domain: IMSDK.getAuthService().getDomain() || '',
                },
            })
        } catch (ex) {
            console.log('xxx', ex)
        }
    }

    const isConvOwner = (conv: any) => {
        return conv?.groupChatUserConversation?.groupChatConversation?.ownerUid === IMSDK.getAuthService().getMyUid() + ''
    }

    const isConvAdmin = (conv: any) => {
        return conv?.groupChatUserConversation?.groupChatConversation?.ownerUid === IMSDK.getAuthService().getMyUid() + ''
    }

    return (

        <Drawer title="群设置" placement="right" onClose={close} visible={appCid !== ''}>
            <div style={{ margin: '10px auto' }}>群ID：{appCid}</div>
            <div style={{ margin: '10px auto' }}>类型：{isGroupConv ? '群聊' : '单聊'}</div>
            {isGroupConv && <div style={{ margin: '10px auto' }}>我是群主：{isConvOwner(conv).toString()}</div>}
            {isGroupConv && (
                <div style={{ margin: '10px auto' }}>群名称：{conv.groupChatUserConversation.groupChatConversation.title} <a href="###" onClick={updateTitle}>修改</a></div>
            )}
            {isGroupConv && (
                <div style={{ margin: '10px auto' }}>群图标：
                    <span>
                        {icon ? <img style={{
                            width: 30,
                            height: 30,
                            borderRadius: '50%',
                        }} src={icon} /> : '无'}
                        &nbsp;
                    </span>
                    <Upload
                        maxCount={1}
                        name="file"
                        showUploadList={false}
                        accept=".jpg, .jpeg, .png, .gif"
                        beforeUpload={(file) => {
                            const fileTypeMap: { [key: string]: number } = {
                                webp: 1,
                                png: 2,
                                jpg: 3,
                                jpeg: 3,
                                gif: 4,
                            }
                            console.log('upload beforeUpload', file)
                            const fileType = fileTypeMap[file.type.split('/')[1]]
                            FileUpload.fragUpload(file, {
                                conversationId: appCid,
                                uuid: msgFeature.genUUid(),
                                onBeforeUpload(uuid: string, fileInfo: any) {
                                    console.log('FileUpload onBeforeUpload', fileInfo)
                                    setIconUpload({
                                        progress: 0,
                                        fileName: fileInfo.name,
                                        base64: fileInfo.base64 || ''
                                    })
                                },
                                onProgress(uuid: string, num: number) {
                                    console.log('FileUpload onProgress', num)
                                    setIconUpload({
                                        ...iconUpload,
                                        progress: num,
                                    })
                                },
                            }).then((res: any) => {
                                console.log('FileUpload succ', res)
                                setIconUpload({})
                                if (res?.mediaId) {
                                    // 上传成功， 更新图标
                                    IMSDK.getConvService().groupConversationUpdateIcon(appCid, res.mediaId).then(() => {
                                        message.info('群图标修改成功')
                                    }).catch((ex: any) => {
                                        if (ex?.body.reason) {
                                            message.error('更新图标失败，原因：' + ex?.body.reason);
                                        }
                                    })
                                }
                            }).catch((ex: any) => {
                                console.log('FileUpload err', ex)
                                setIconUpload({})
                            })
                            return false;
                        }}>
                        <Button size="small">+ 上传</Button>
                        {iconUpload.progress && <span>上传中 {iconUpload.progress}%</span>}
                    </Upload>
                </div>
            )}
            {isGroupConv && (
                <div style={{ margin: '10px auto' }}>
                    群成员：{groupMembers.map((item: any) => <Tag visible={true} closable={item.role !== 1} key={item.uid} onClose={() => {
                        // 群踢人
                        console.log(item.uid)
                        IMSDK.getConvService().groupConversationKick({
                            appCid,
                            users: [item.uid],
                        }).catch((ex: any) => {
                            message.error(ex?.body?.reason)
                        })
                    }}>{
                            (item.groupNick || item.uid) + '-' + GroupRoleMap[item.role]
                        }</Tag>)}
                    <Tag onClick={() => {
                        const id = window.prompt('输入需添加的用户ID，以“,”分隔，test001-test999')
                        IMSDK.getConvService().groupConversationJoin({
                            appCid,
                            initMembers: id?.split(',').map((id) => (
                                {
                                    user: id,
                                    role: 3,
                                }
                            ))
                        }).catch((ex: any) => {
                            message.error(ex?.body?.reason)
                        })
                    }} style={{ cursor: 'pointer' }}>
                        + 添加
                    </Tag>
                </div>
            )}

        </Drawer>
    );
});
