import React, { useState, useCallback, FunctionComponent, useEffect } from 'react';
import { Button, Dropdown, Input, Menu, Modal, Form, Layout } from 'antd';

import './Topbar.less';


declare const AIM: any;
const FormItem = Form.Item;
const {
    IMSDK, // 核心 SDK
    DataSDK, // 数据 SDK
    constants, // 常量定义
    convFeature, // 会话相关辅助函数
    msgFeature, // 消息相关辅助函数
    initStore, // 数据 SDK，初始化 Store
    getFileUrl,  // 获取文件访问地址
    mediaId, // 资源 ID 工具,
    ConnectionStatus,
    EventNameEnum,
} = AIM

export interface IProps {
    loginStatus: Number;
    logout: () => void;
}

enum ModalType {
    singleConv,
    groupConv,
}

export const Topbar: FunctionComponent<IProps> = React.memo((props: IProps) => {
    const { loginStatus, logout } = props;
    const [currentModal, setCurrentModal] = useState<ModalType>()
    const [currentModalLoading, setCurrentModalLoading] = useState<ModalType>()
    const [formSingleConv] = Form.useForm();
    const [formGroupConv] = Form.useForm();
    const menu = (
        <Menu>
            <Menu.Item onClick={() => { setCurrentModal(ModalType.singleConv); }}>
                创建单聊
            </Menu.Item>
            <Menu.Item onClick={() => { setCurrentModal(ModalType.groupConv); }}>
                创建群聊
            </Menu.Item>
        </Menu>
    );

    return (
        <Layout.Header className="layout-header">
            <div className="title">DEMO</div>
            {loginStatus === ConnectionStatus.CONN_AUTHED && (
                <Button style={{ marginLeft: 'auto', marginRight: 20 }} type="default" onClick={logout} size="small">退出</Button>
            )}
            {loginStatus === ConnectionStatus.CONN_AUTHED && (
                <Dropdown overlay={menu} placement="bottomLeft" arrow>
                    <Button type="default" ghost size="small">add</Button>
                </Dropdown>
            )}
            <Modal
                title="创建单聊"
                visible={currentModal === ModalType.singleConv}
                onOk={() => {
                    formSingleConv
                        .validateFields()
                        .then(values => {
                            // formSingleConv.resetFields();
                            if (values?.uid) {
                                const domain = IMSDK.getCloudVersion() ? (`@${IMSDK.getAuthService().getDomain()}`) : ''
                                const [pairFirst, pairSecond] = [
                                    IMSDK.getAuthService().getMyUid(),
                                    values?.uid
                                ].sort((a, b) => parseInt(a, 10) - parseInt(b, 10))
                                IMSDK.getConvService().createSingleConversation({
                                    appCid: `$1$${pairFirst}:${pairSecond}`,
                                    pairFirst,
                                    pairSecond,
                                    extension: {},
                                }).then((res: any) => {
                                    setCurrentModal(undefined);
                                    if(res.singleChatConversation && res.singleChatConversation.appCid) {
                                        DataSDK.activeConv(res.singleChatConversation.appCid)
                                    }
                                })
                            }
                        })
                        .catch(info => {
                            console.log('Validate Failed:', info);
                        });
                }}
                confirmLoading={currentModalLoading === ModalType.singleConv}
                onCancel={() => { setCurrentModal(undefined) }}
            >
                <Form
                    form={formSingleConv}
                    layout="vertical"
                    name="form_in_modal"
                >
                    <Form.Item
                        name="uid"
                        label="对方 ID, test001-test999"
                        rules={[{ required: true, message: '对方 ID, test001-test999' }]}
                    >
                        <Input placeholder="对方 ID, test001-test999" />
                    </Form.Item>
                    <div>说明文档：<a href="https://help.aliyun.com/document_detail/209041.html" target="_blank">标准单聊</a></div>
                </Form>
            </Modal>
            <Modal
                title="创建群聊"
                visible={currentModal === ModalType.groupConv}
                onOk={() => {
                    formGroupConv
                        .validateFields()
                        .then(values => {
                            // formSingleConv.resetFields();
                            if (values?.uids) {
                                console.log(111, values?.uids)
                                const uids = values?.uids.split(',')
                                const initMembers = uids.map((id: string) => ({
                                    user: id,
                                    role: 3, // 1-群主，2-管理员，3-普通
                                    // joinTime: Date.now(),
                                    groupNick: '',
                                    extension: {},
                                }))

                                IMSDK.getConvService().groupConversationCreate({
                                    uuid: msgFeature.genUUid(),
                                    initMembers,
                                    defaultTitle: values.title,
                                    iconResource: '',
                                    extension: {},
                                }).then((res: any) => {
                                    setCurrentModal(undefined);
                                    if(res.singleChatConversation && res.singleChatConversation.appCid) {
                                        DataSDK.activeConv(res.singleChatConversation.appCid)
                                    }
                                })
                            }
                        })
                        .catch(info => {
                            console.log('Validate Failed:', info);
                        });
                }}
                confirmLoading={currentModalLoading === ModalType.groupConv}
                onCancel={() => { setCurrentModal(undefined) }}
            >
                <Form
                    form={formGroupConv}
                    layout="vertical"
                    name="form_in_modal"
                >
                    <Form.Item
                        name="title"
                        label="群名称"
                        rules={[{ required: true, message: '请输入群名称' }]}
                    >
                        <Input placeholder="请输入群名称" />
                    </Form.Item>
                    <Form.Item
                        name="uids"
                        label="群成员ID列表"
                        rules={[{ required: true, message: '请输入群成员ID，英文逗号分隔。test001-test009' }]}
                    >
                        <Input placeholder="请输入群成员ID，英文逗号分隔。test001-test009" />
                    </Form.Item>
                </Form>
            </Modal>
        </Layout.Header>
    );
});
